// This file describes the API available to user application to control the
// NFSP daemon running locally
//
// Author: NFSP consortium 8)
// Date: 2002/07
// Licence: GPLv2

#ifndef __NFSPDCTL_H__
#define __NFSPDCTL_H__ 1

#define NFSPDCTL_API_VERSION 100

#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>

// opens the connection with the nfspd
//
// return:
//  -1 error
//  0  success
int nfspd_initialize();

// closes the connection with the nfspd
//
// return:
//  -1 error
//   0 success
int nfspd_terminate();

// creates a new iod descriptor given its address and port
// 
// Return:
//  >=0 iod_handler
//  <0  error
int nfspd_addiod(const struct sockaddr_in *addr);

// free an iod descriptor
//
// return:
//  0   success
//  <0  -errno
int nfspd_freeiod(const int ih);

// bind an iod to a viod
//
// requisite:
//  ih     exists (nfspd_addiod)
//  iviod  must be in 0..nbviods-1
// 
// return:
//  0   success
//  <0  -errno
int nfspd_bindiod(int iviod, int ih);

// unbind an iod to a viod
//
// requisite:
//  ih     exists (nfspd_addiod)
//  iviod  must be in 0..nbviods-1
//  ih must be binded to iviod
// 
// return:
//  0   success
//  <0  -errno
int nfspd_unbindiod(int ih);

// sets nbviod
//
// nb >=0
//
// return:
//  0   success
//  <0  -errno
int nfspd_setnbviods(int nb);

// gets the viods number
// 
// return:
//  >=0  nb of viods (success)
//  <0   -errno
int nfspd_getnbviods();

// asks the nfspd to go in DROP mode
//
// return:
//  <0  error
//   0   success
int nfspd_drop();

// asks the nfspd to go in FORWARD mode
//
// return:
//  <0  error
//   0   success
int nfspd_forward();

// asks the nfspd to reset its mode (a.k.a. FLUSH operation)
//
// return:
//  <0  error
//   0   success
int nfspd_flush();

// ask the nfspd how many iods are in the viod
// 
// return:
//  <0 indicates an error
//   0 indicates no iod left in this viod
//  >0 there is at least 1 iod in this viod
//     NB: it currently always return 1 even though there are more iods
//
// NB: sticking to the 3 cases <0, ==0 and >0 is safer than expecting to have
// the realnumber.
int nfspd_get_viod_size(const int iviod);

#endif

