/*
 * Author: pl 200211xx
 * License: GPLv2
 *
 *  WARNING: these operations are not multithread-safe when used concurrently on
 * a same file descriptor
 * */

#ifdef MSWIN
#include <unistd.h>
#include <unistd.h>

#include "pread_pwrite.h"

#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

//off_t lseek(int fildes, off_t offset, int whence);
//ssize_t read(int fd, void *buf, size_t count);
//ssize_t write(int fd, const void *buf, size_t count);


ssize_t pread(int fd, void *buf, size_t count, off_t offset) {
	off_t old_off;
	off_t new_off;
	ssize_t bytesread;

	old_off = lseek(fd, 0, SEEK_CUR);
	if (old_off == (off_t)-1) {
		bytesread = -1;
		goto restore_and_exit;
	}

	new_off = lseek(fd, offset, SEEK_SET);
	if (new_off != offset) {
		bytesread = -1;
		goto restore_and_exit;
	}

	bytesread = read(fd, buf, count);

restore_and_exit:
	if ((off_t)-1 == lseek(fd, old_off, SEEK_SET)) {
		bytesread = -1;
	}

	return bytesread;

}

ssize_t pwrite(int fd, void *buf, size_t count, off_t offset) {
	off_t old_off;
	off_t new_off;
	ssize_t byteswritten;

	old_off = lseek(fd, 0, SEEK_CUR);
	if (old_off == (off_t)-1) {
		byteswritten = -1;
		goto restore_and_exit;
	}

	new_off = lseek(fd, offset, SEEK_SET);
	if (new_off != offset) {
		byteswritten = -1;
		goto restore_and_exit;
	}

	byteswritten = write(fd, buf, count);

restore_and_exit:
	if ((off_t)-1 == lseek(fd, old_off, SEEK_SET)) {
		byteswritten = -1;
	}

	return byteswritten;

}

#endif /* MSWIN */

