// Author: Pierre Lombard <pierre.lombard@imag.fr>
// Date: 200211xx
// License: GPLv2

#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "recursdir.h"
#include "xalloc.h"

#include "list.h"

#include "Xstat.h"

#undef TEST
#undef VERBOSE

struct file_hdl_t {
	struct valid_file_t vf;
	struct list_head L;
};

struct dir_hdl_t {
	char *filename;
	struct list_head L;
};

/**
 * mk_valid_inode_list - creates a file list of file_hdl_t 
 * @rootdir:		the directory to be wandered
 * @list_filesp:	the head of the list when returned
 * @list_sizep:		the list size when returned
 */
void
mk_valid_inode_list(char *rootdir, struct list_head *list_filesp,
		    int *list_sizep)
{
	int ret;
	char *curdir;
	LIST_HEAD(fifo_dirs);

	*list_sizep = 0;

	if (!rootdir) {
		return;
	}

	//printf("\n*** mk_valid_inode_list ***\n");

	{
		struct dir_hdl_t *itemp = xmalloc(sizeof (*itemp));
		itemp->filename = strdup(".");
		INIT_LIST_HEAD(&itemp->L);

		list_add_tail(&itemp->L, &fifo_dirs);
	}

	while (!list_empty(&fifo_dirs)) {
		struct dirent **namelist;
		struct dir_hdl_t *old;
		int n;

		old = list_entry(fifo_dirs.next, struct dir_hdl_t, L);

		curdir =
		    xmalloc(strlen(rootdir) + 1 + strlen(old->filename) + 1);
		strcpy(curdir, rootdir);
		strcat(curdir, "/");
		strcat(curdir, old->filename);
		//printf("BROWSING dir: %s\n", curdir);

		{

			n = scandir(curdir, &namelist, 0, NULL);
			if (n < 0) {
				fprintf(stderr, "cannot scandir\n");
			} else {
				while (n--) {
					struct Xstat xstatbuf;
					struct stat * statbufp = &xstatbuf.mfstat;
					char *fullpath;

					fullpath =
					    xmalloc(strlen(curdir) + 1 +
						    strlen(namelist[n]->d_name) + 1);
					strcpy(fullpath, curdir);
					strcat(fullpath, "/");
					strcat(fullpath, namelist[n]->d_name);

					ret = Xlstat(fullpath, &xstatbuf);
					if (ret == -1) {
						fprintf(stderr,
							"WARNING: lstat(%s) failed errno=%d\n",
							fullpath, errno);
						xfree(namelist[n]);
						xfree(fullpath);
						continue;
					}
					if (S_ISREG(statbufp->st_mode)) {
						struct file_hdl_t *itemp;

						//printf("%s\n", fullpath); //, statbufp->st_ino);
						//printf("ENQUEUED dir: %s [fp=%s]\n", namelist[n]->d_name, fullpath);

						itemp = xmalloc(sizeof (*itemp));

						itemp->vf.inode = statbufp->st_ino;
						itemp->vf.seed = xstatbuf.nstat.seed;

						xfree(namelist[n]);
						INIT_LIST_HEAD(&itemp->L);
						// It does not matter whether we add to head or tail
						list_add(&itemp->L,
							 list_filesp);
						(*list_sizep)++;

#ifdef VERBOSE
						if ((*list_sizep) % 50 == 0) {
							printf
							    ("\b\b\b\b\b\b\b\b\b\b\b\b\b\b%8d files",
							     *list_sizep);
							fflush(stdout);
						}
#endif				/* VERBOSE */

					} else if (S_ISDIR(statbufp->st_mode)) {
						if (!strcmp
						    (namelist[n]->d_name, ".")
						    || !strcmp(namelist[n]->
							       d_name, "..")) {
							//printf("skipping %s\n", namelist[n]->d_name);
							xfree(namelist[n]);
						} else {
							struct dir_hdl_t *itemp;
							//printf("%s/\n", fullpath);
							//printf("ENQUEUED dir: %s [fp=%s]\n", namelist[n]->d_name, fullpath);

							itemp = xmalloc(sizeof (*itemp));
							itemp->filename =
							    strdup(fullpath + strlen (rootdir) + 1);
							xfree(namelist[n]);
							INIT_LIST_HEAD(&itemp->L);
							list_add_tail(&itemp->L, &fifo_dirs);
						}
					} else {
						// not a regular nor a directory
					}
					xfree(fullpath);
				}
				xfree(namelist);
			}

		}

		xfree(curdir);
		list_del(&old->L);
		xfree(old->filename);
		xfree(old);
	}

#ifdef VERBOSE
	printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b%8d files", *list_sizep);
	fflush(stdout);
#endif				/* VERBOSE */
}

/**
 * mk_valid_inode_array - creates a file list of file_hdl_t 
 * @rootdir:		the directory to be wandered
 * @file_arrayp:	(out) returns a malloc'ated array in *file_arrayp
 * @file_array_sizep:	(out) size of the array is *file_array_sizep
 */
void
mk_valid_inode_array(char *rootdir, struct valid_file_t **file_arrayp,
		     int *file_array_sizep)
{
	struct valid_file_t *file_array;
	int file_array_size;
	int list_size;
	LIST_HEAD(list_files);
	struct list_head *pos, *tmp;
	int i = 0;

	//printf("\n*** mk_valid_inode_array\n");

	mk_valid_inode_list(rootdir, &list_files, &list_size);

	// Convert the list to an array + free the list
	file_array_size = list_size;
	file_array = xmalloc(list_size * sizeof (*file_array));

	list_for_each_safe(pos, tmp, &list_files) {
		struct file_hdl_t *itemp =
		    list_entry(pos, struct file_hdl_t, L);
		file_array[i].inode = itemp->vf.inode;
		file_array[i].seed = itemp->vf.seed;
		printf("(%lx,%lx)\n", itemp->vf.inode, itemp->vf.seed);
		i++;
		xfree(itemp);
	}

	*file_arrayp = file_array;
	*file_array_sizep = file_array_size;
}

#ifdef TEST
int
main(int argc, char *argv[])
{
	int list_size;
	LIST_HEAD(list_files);

	struct valid_file_t *file_array;
	int file_array_size;

#ifdef VERBOSE
	// Get the files list
	printf("Getting file list... " "              ");
	fflush(stdout);
	mk_valid_inode_list(argv[1], &list_files, &list_size);

	printf("\n");
	fflush(stdout);
#endif				/* VERBOSE */

	// Convert the list to an array + free the list
	file_array_size = list_size;
	file_array = xmalloc(list_size * sizeof (*file_array));
	{
		struct list_head *pos, *tmp;
		int i = 0;
		list_for_each_safe(pos, tmp, &list_files) {
			struct file_hdl_t *itemp =
			    list_entry(pos, struct file_hdl_t, L);
			file_array[i].inode = itemp->vf.inode;
			file_array[i].seed = itemp->vf.seed;
			i++;
			//printf("(%lx,%lx)\n", itemp->inode, itemp->seed);
			xfree(itemp);
		}
	}

	// Free the files array
	xfree(file_array);

	return 0;
}
#endif				/* TEST */
